import { Logger } from '@hackler/javascript-sdk';

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

var DEVTOOLS_URL = "https://devtools.hackle.io";
var DEVTOOLS_USER_EXPLORER_SUB_PATH = "/user-explorer";
var HACKLE_LOGO_URL = "https://cdn.hackle.io/images/20230330/84a95e8b-339b-412e-9200-f3655b5eb91b.png";

var $ = function (selector, from) {
    var _from = from || document;
    var elem = _from.querySelector(selector);
    if (!elem)
        throw Error("HackleDevtools Failed when getting element.");
    return elem;
};
var rootId = "hackle-devtools";
var togglerId = "hackle-devtools__toggler";
var shadowWrapperId = "hackle-devtools__shadowWrapper";
var iframeWrapperId = "hackle-devtools__iframeWrapper";
var iframeId = "hackle-devtools__iframe";
var stylesheetId = "hackle-devtools__style";
var iframeWrapper = "\n  <template id=\"".concat(iframeWrapperId, "-template\">\n    <div id=\"").concat(iframeWrapperId, "\">\n      <iframe id=\"").concat(iframeId, "\"></iframe>\n    </div>\n  </template>\n");
var shadowWrapper = "\n  <template id=\"".concat(shadowWrapperId, "-template\">\n    <div id=\"").concat(shadowWrapperId, "\" style=\"position:fixed; z-index:10000001; left:24px; bottom: 61px; height: 0;\"></div>\n  </template>\n");
var toggler = "\n  <template id=\"".concat(togglerId, "-template\">\n    <button id=\"").concat(togglerId, "\" type=\"button\">\n    </button>\n  </template>\n\n");
var stylesheet = "\n  <template id=\"".concat(stylesheetId, "-template\">\n    <style id=\"").concat(stylesheetId, "\">\n      #").concat(togglerId, " {\n        width: 45px;\n        height: 45px;\n        display: block;\n        background-image: url(").concat(HACKLE_LOGO_URL, ");\n        background-size: contain;\n        background-color: white;\n\n        border: 2px solid white;\n        border-radius: 50%;\n        cursor: pointer;\n        box-shadow: 0px 0px 10px 3px #CACACA;\n      }\n      #").concat(togglerId, ".hide {\n        display: none;\n      }\n      #").concat(togglerId, " > img {\n        width: 100%;\n        height: 100%;\n      }\n      #").concat(iframeId, " {\n        width: 100%;\n        height: 100%;\n        border: none;\n        border-radius: 8px;\n      }\n      #").concat(iframeWrapperId, " {\n        width:420px;\n        height:80%;\n        min-height:580px;\n        max-height:620px;\n        border-radius:18px;\n        position:fixed;\n        left:24px;\n        bottom:18px;\n        animation: fade 0.5s;\n        background-color: #F5F5F5;\n        overflow: hidden;\n        box-shadow: rgba(255, 255, 255, 0.12) 0px 0px 2px 0px inset, rgba(0, 0, 0, 0.05) 0px 0px 2px 1px, rgba(0, 0, 0, 0.3) 0px 12px 60px;\n      }\n\n      @media screen and (max-width: 768px) {\n        #").concat(iframeWrapperId, " {\n          width: 100vw;\n          min-height: unset;\n          max-height: unset;\n          left: 0;\n        }\n      }\n      @keyframes fade {\n        from {\n            opacity: 0;\n        }\n        to {\n            opacity: 1;\n        }\n      }\n    </style>\n  </template>\n");
var templates = {
    iframeWrapper: iframeWrapper,
    shadowWrapper: shadowWrapper,
    toggler: toggler,
    stylesheet: stylesheet
};

var MessageEventType;
(function (MessageEventType) {
    MessageEventType["INIT"] = "HACKLE-EVENT_INIT";
    MessageEventType["CLOSE"] = "HACKLE-EVENT_CLOSE";
    MessageEventType["USER"] = "HACKLE-EVENT_USER";
    MessageEventType["DECISION_AB_GET"] = "HACKLE-EVENT_GET_AB_DECISION";
    MessageEventType["DECISION_FF_GET"] = "HACKLE-EVENT_GET_FF_DECISION";
    MessageEventType["OVERRIDE_AB_GET"] = "HACKLE-EVENT_GET_AB_OVERRIDE";
    MessageEventType["OVERRIDE_AB_SET"] = "HACKLE-EVENT_SET_AB_OVERRIDE";
    MessageEventType["OVERRIDE_AB_RESET"] = "HACKLE-EVENT_RESET_AB_OVERRIDE";
    MessageEventType["OVERRIDE_AB_RESET_ALL"] = "HACKLE-EVENT_RESET_ALL_AB_OVERRIDE";
    MessageEventType["OVERRIDE_FF_GET"] = "HACKLE-EVENT_GET_FF_OVERRIDE";
    MessageEventType["OVERRIDE_FF_SET"] = "HACKLE-EVENT_SET_FF_OVERRIDE";
    MessageEventType["OVERRIDE_FF_RESET"] = "HACKLE-EVENT_RESET_FF_OVERRIDE";
    MessageEventType["OVERRIDE_FF_RESET_ALL"] = "HACKLE-EVENT_RESET_ALL_FF_OVERRIDE";
})(MessageEventType || (MessageEventType = {}));
var isDevtoolsMessage = function (message) {
    return typeof message.type === "string" && Object.values(MessageEventType).includes(message.type);
};
var isCloseMessage = function (message) {
    return isDevtoolsMessage(message) && message.type === MessageEventType.CLOSE;
};
var isInitMessage = function (message) {
    return isDevtoolsMessage(message) && message.type === MessageEventType.INIT;
};
var isUserMessage = function (message) {
    return isDevtoolsMessage(message) && message.type === MessageEventType.USER;
};
var isGetABDecisionMessage = function (message) {
    return isDevtoolsMessage(message) && message.type === MessageEventType.DECISION_AB_GET;
};
var isGetFFDecisionMessage = function (message) {
    return isDevtoolsMessage(message) && message.type === MessageEventType.DECISION_FF_GET;
};
var isGetABOverrideMessage = function (message) {
    return isDevtoolsMessage(message) && message.type === MessageEventType.OVERRIDE_AB_GET;
};
var isGetFFOverrideMessage = function (message) {
    return isDevtoolsMessage(message) && message.type === MessageEventType.OVERRIDE_FF_GET;
};
var isSetABOverrideMessage = function (message) {
    return isDevtoolsMessage(message) && message.type === MessageEventType.OVERRIDE_AB_SET;
};
var isSetFFOverrideMessage = function (message) {
    return isDevtoolsMessage(message) && message.type === MessageEventType.OVERRIDE_FF_SET;
};
var isResetABOverrideMessage = function (message) {
    return isDevtoolsMessage(message) && message.type === MessageEventType.OVERRIDE_AB_RESET;
};
var isResetFFOverrideMessage = function (message) {
    return isDevtoolsMessage(message) && message.type === MessageEventType.OVERRIDE_FF_RESET;
};
var isResetAllABOverrideMessage = function (message) {
    return isDevtoolsMessage(message) && message.type === MessageEventType.OVERRIDE_AB_RESET_ALL;
};
var isResetAllFFOverrideMessage = function (message) {
    return isDevtoolsMessage(message) && message.type === MessageEventType.OVERRIDE_FF_RESET_ALL;
};

var TransceiverImpl = /** @class */ (function () {
    function TransceiverImpl(port, opposite) {
        this.port = port;
        this.opposite = opposite;
        this.port.start();
    }
    TransceiverImpl.prototype.receive = function (messageFilter, listener) {
        this.port.addEventListener("message", function (e) {
            if (messageFilter(e.data)) {
                listener(e.data.payload);
            }
        });
    };
    TransceiverImpl.prototype.post = function (message) {
        this.port.postMessage(message);
    };
    TransceiverImpl.prototype.close = function () {
        this.port.close();
        this.opposite.close();
    };
    return TransceiverImpl;
}());

var log$1 = Logger.log;
var CHANNEL_INIT_TIMEOUT = 5000;
var MessageChannelManager = /** @class */ (function () {
    function MessageChannelManager(transceiver) {
        this.transceiver = transceiver;
    }
    MessageChannelManager.connectTo = function (iframe, path) {
        return __awaiter(this, void 0, void 0, function () {
            var timer, timeoutResolver, timeoutPromise, _a, port1, port2, transceiver;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        timeoutPromise = new Promise(function (resolve) {
                            timeoutResolver = resolve;
                            timer = setTimeout(function () {
                                resolve({ port1: undefined, port2: undefined });
                            }, CHANNEL_INIT_TIMEOUT);
                        });
                        window.addEventListener("message", function (e) {
                            if (isInitMessage(e.data) && iframe.contentWindow) {
                                var _a = new MessageChannel(), port1_1 = _a.port1, port2_1 = _a.port2;
                                iframe.contentWindow.postMessage({
                                    type: MessageEventType.INIT
                                }, "".concat(DEVTOOLS_URL, "/").concat(path), [port2_1]);
                                clearTimeout(timer);
                                timeoutResolver === null || timeoutResolver === void 0 ? void 0 : timeoutResolver({ port1: port1_1, port2: port2_1 });
                            }
                        });
                        return [4 /*yield*/, timeoutPromise];
                    case 1:
                        _a = _b.sent(), port1 = _a.port1, port2 = _a.port2;
                        if (!port1 || !port2) {
                            log$1.error("Timeout occurred while loading devtools.");
                            return [2 /*return*/];
                        }
                        transceiver = new TransceiverImpl(port1, port2);
                        return [2 /*return*/, new MessageChannelManager(transceiver)];
                }
            });
        });
    };
    MessageChannelManager.prototype.addMessageListener = function (messageFilter, listener) {
        this.transceiver.receive(messageFilter, listener);
    };
    MessageChannelManager.prototype.applyChannel = function (channel) {
        channel(this.transceiver);
    };
    MessageChannelManager.prototype.close = function () {
        this.transceiver.close();
    };
    return MessageChannelManager;
}());

var createUserExplorerChannel = function (userExplorer) { return function (transceiver) {
    /**
     * User Messages
     */
    transceiver.receive(isUserMessage, function () {
        var response = {
            type: MessageEventType.USER,
            payload: userExplorer.currentUser()
        };
        transceiver.post(response);
    });
    /**
     * A/B Test Messages
     */
    transceiver.receive(isGetABDecisionMessage, function () {
        var response = {
            type: MessageEventType.DECISION_AB_GET,
            payload: userExplorer.getAbTestDecisions()
        };
        transceiver.post(response);
    });
    transceiver.receive(isGetABOverrideMessage, function () {
        var response = {
            type: MessageEventType.OVERRIDE_AB_GET,
            payload: userExplorer.getAbTestOverrides()
        };
        transceiver.post(response);
    });
    transceiver.receive(isSetABOverrideMessage, function (payload) {
        if (!payload)
            return;
        userExplorer.setAbTestOverride(payload.experiment, payload.variationId);
    });
    transceiver.receive(isResetABOverrideMessage, function (payload) {
        if (!payload)
            return;
        userExplorer.resetAbTestOverride(payload.experiment);
    });
    transceiver.receive(isResetAllABOverrideMessage, function () {
        userExplorer.resetAllAbTestOverride();
    });
    /**
     * Feature Flag Messages
     */
    transceiver.receive(isGetFFDecisionMessage, function () {
        var response = {
            type: MessageEventType.DECISION_FF_GET,
            payload: userExplorer.getFeatureFlagDecisions()
        };
        transceiver.post(response);
    });
    transceiver.receive(isGetFFOverrideMessage, function () {
        var response = {
            type: MessageEventType.OVERRIDE_FF_GET,
            payload: userExplorer.getFeatureFlagOverrides()
        };
        transceiver.post(response);
    });
    transceiver.receive(isSetFFOverrideMessage, function (payload) {
        if (!payload)
            return;
        userExplorer.setFeatureFlagOverride(payload.experiment, payload.variationId);
    });
    transceiver.receive(isResetFFOverrideMessage, function (payload) {
        if (!payload)
            return;
        userExplorer.resetFeatureFlagOverride(payload.experiment);
    });
    transceiver.receive(isResetAllFFOverrideMessage, function () {
        userExplorer.resetAllFeatureFlagOverride();
    });
}; };

var makeDraggable = function (target) {
    var isPress = false, prevPosX = 0, prevPosY = 0;
    var start = function (e) {
        if (isPress)
            return;
        var touch = e instanceof TouchEvent ? e.touches[0] : e;
        prevPosX = touch.clientX;
        prevPosY = touch.clientY;
        isPress = true;
        // Desktop
        window.addEventListener("mousemove", move);
        // Mobile
        window.addEventListener("touchmove", move, { passive: false });
    };
    var move = function (e) {
        if (!isPress)
            return;
        target.classList.add("dragged");
        var touch = e instanceof TouchEvent ? e.touches[0] : e;
        var posX = prevPosX - touch.clientX;
        var posY = prevPosY - touch.clientY;
        prevPosX = touch.clientX;
        prevPosY = touch.clientY;
        target.style.left = target.offsetLeft - posX + "px";
        target.style.top = target.offsetTop - posY + "px";
        e.preventDefault();
    };
    var end = function () {
        isPress = false;
        // Desktop
        window.removeEventListener("mousemove", move);
        // Mobile
        window.removeEventListener("touchmove", move);
    };
    // Desktop
    target.addEventListener("mousedown", start);
    target.addEventListener("mouseup", end);
    // Mobile
    target.addEventListener("touchstart", start, { passive: false });
    target.addEventListener("touchend", end);
};

var log = Logger.log;
var DevtoolManager = /** @class */ (function () {
    function DevtoolManager(devtoolSubPath, channel) {
        this.devtoolSubPath = devtoolSubPath;
        this.channel = channel;
        this.channelManager = null;
        this.shadowRoot = undefined;
        this.initialize();
    }
    DevtoolManager.prototype.getShadowWrapper = function (from) {
        return $("#".concat(shadowWrapperId), from);
    };
    DevtoolManager.prototype.getIFrameWrapper = function (from) {
        return $("#".concat(iframeWrapperId), from);
    };
    DevtoolManager.prototype.getToggler = function (from) {
        return $("#".concat(togglerId), from);
    };
    DevtoolManager.prototype.getIFrame = function (from) {
        return $("#".concat(iframeId), from);
    };
    DevtoolManager.prototype.getTemplate = function (elementId) {
        return $("#".concat(elementId, "-template"));
    };
    DevtoolManager.prototype.createMessageChannel = function () {
        return __awaiter(this, void 0, void 0, function () {
            var iframe, channelManager;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        iframe = this.getIFrame(this.shadowRoot);
                        iframe.src = "".concat(DEVTOOLS_URL, "/").concat(this.devtoolSubPath);
                        return [4 /*yield*/, MessageChannelManager.connectTo(iframe, this.devtoolSubPath)];
                    case 1:
                        channelManager = _a.sent();
                        if (!channelManager) {
                            this.hide();
                            return [2 /*return*/];
                        }
                        this.channelManager = channelManager;
                        channelManager.addMessageListener(isCloseMessage, this.hide.bind(this));
                        channelManager.applyChannel(this.channel);
                        return [2 /*return*/];
                }
            });
        });
    };
    DevtoolManager.prototype.initialize = function () {
        if (typeof window === "undefined") {
            log.error("HackleDevtools called before window loaded.");
            return;
        }
        if (document.getElementById(rootId)) {
            return;
        }
        this.createRoot();
    };
    DevtoolManager.prototype.show = function () {
        var _a;
        var iframeWrapperTemplate = this.getTemplate(iframeWrapperId);
        (_a = this.shadowRoot) === null || _a === void 0 ? void 0 : _a.appendChild(iframeWrapperTemplate.content.cloneNode(true));
        this.createMessageChannel();
    };
    DevtoolManager.prototype.createRoot = function () {
        var _this = this;
        var root = document.createElement("div");
        root.id = "hackle-devtools";
        Object.values(templates).forEach(function (template) { return root.insertAdjacentHTML("beforeend", template); });
        document.body.appendChild(root);
        var shadowWrapperTemplate = this.getTemplate(shadowWrapperId);
        root.appendChild(shadowWrapperTemplate.content);
        var shadowWrapperElement = this.getShadowWrapper();
        makeDraggable(shadowWrapperElement);
        var shadowDOM = shadowWrapperElement.attachShadow({ mode: "open" });
        this.shadowRoot = shadowDOM;
        var togglerTemplate = this.getTemplate(togglerId);
        var stylesheet = this.getTemplate(stylesheetId);
        shadowDOM.appendChild(stylesheet.content);
        shadowDOM.appendChild(togglerTemplate.content);
        var toggler = this.getToggler(shadowDOM);
        toggler.addEventListener("click", function () {
            if (shadowWrapperElement.classList.contains("dragged")) {
                shadowWrapperElement.classList.remove("dragged");
                return;
            }
            _this.show();
            toggler.classList.add("hide");
        }, true);
    };
    DevtoolManager.prototype.hide = function () {
        var toggler = this.getToggler(this.shadowRoot);
        toggler.classList.remove("hide");
        this.getIFrameWrapper(this.shadowRoot).remove();
    };
    DevtoolManager.prototype.close = function () {
        var _a;
        try {
            (_a = this.channelManager) === null || _a === void 0 ? void 0 : _a.close();
            var root = $("#".concat(rootId));
            root.remove();
        }
        catch (error) {
            log.error("Failed to close HackleDevtools.");
        }
    };
    return DevtoolManager;
}());
var HackleDevTools = /** @class */ (function () {
    function HackleDevTools() {
    }
    HackleDevTools.userExplorer = function (userExplorer) {
        try {
            HackleDevTools.devtoolInstance = new DevtoolManager(DEVTOOLS_USER_EXPLORER_SUB_PATH, createUserExplorerChannel(userExplorer));
        }
        catch (error) {
            log.error("Failed to initialize HackleDevTools.");
        }
    };
    HackleDevTools.close = function () {
        if (!HackleDevTools.devtoolInstance) {
            log.error("There is no active HackleDevtools.");
            return;
        }
        HackleDevTools.devtoolInstance.close();
    };
    HackleDevTools.devtoolInstance = null;
    return HackleDevTools;
}());

export { MessageEventType, HackleDevTools as default, isCloseMessage, isDevtoolsMessage, isGetABDecisionMessage, isGetABOverrideMessage, isGetFFDecisionMessage, isGetFFOverrideMessage, isInitMessage, isResetABOverrideMessage, isResetAllABOverrideMessage, isResetAllFFOverrideMessage, isResetFFOverrideMessage, isSetABOverrideMessage, isSetFFOverrideMessage, isUserMessage };
//# sourceMappingURL=index.browser.es.js.map
