import { Experiment, HackleUser, HackleUserExplorerBase } from "@hackler/javascript-sdk";
export declare enum MessageEventType {
    INIT = "HACKLE-EVENT_INIT",
    CLOSE = "HACKLE-EVENT_CLOSE",
    USER = "HACKLE-EVENT_USER",
    DECISION_AB_GET = "HACKLE-EVENT_GET_AB_DECISION",
    DECISION_FF_GET = "HACKLE-EVENT_GET_FF_DECISION",
    OVERRIDE_AB_GET = "HACKLE-EVENT_GET_AB_OVERRIDE",
    OVERRIDE_AB_SET = "HACKLE-EVENT_SET_AB_OVERRIDE",
    OVERRIDE_AB_RESET = "HACKLE-EVENT_RESET_AB_OVERRIDE",
    OVERRIDE_AB_RESET_ALL = "HACKLE-EVENT_RESET_ALL_AB_OVERRIDE",
    OVERRIDE_FF_GET = "HACKLE-EVENT_GET_FF_OVERRIDE",
    OVERRIDE_FF_SET = "HACKLE-EVENT_SET_FF_OVERRIDE",
    OVERRIDE_FF_RESET = "HACKLE-EVENT_RESET_FF_OVERRIDE",
    OVERRIDE_FF_RESET_ALL = "HACKLE-EVENT_RESET_ALL_FF_OVERRIDE"
}
export interface DevtoolsMessage {
    type: MessageEventType;
    payload?: unknown;
}
export interface InitMessage extends DevtoolsMessage {
    type: MessageEventType.INIT;
}
export interface CloseMessage extends DevtoolsMessage {
    type: MessageEventType.CLOSE;
}
export interface UserMessage extends DevtoolsMessage {
    type: MessageEventType.USER;
    payload?: HackleUser;
}
export interface GetABDecisionMessage extends DevtoolsMessage {
    type: MessageEventType.DECISION_AB_GET;
    payload?: ReturnType<HackleUserExplorerBase["getAbTestDecisions"]>;
}
export interface GetFFDecisionMessage extends DevtoolsMessage {
    type: MessageEventType.DECISION_FF_GET;
    payload?: ReturnType<HackleUserExplorerBase["getFeatureFlagDecisions"]>;
}
export interface GetABOverrideMessage extends DevtoolsMessage {
    type: MessageEventType.OVERRIDE_AB_GET;
    payload?: ReturnType<HackleUserExplorerBase["getAbTestOverrides"]>;
}
export interface GetFFOverrideMessage extends DevtoolsMessage {
    type: MessageEventType.OVERRIDE_FF_GET;
    payload?: ReturnType<HackleUserExplorerBase["getFeatureFlagOverrides"]>;
}
export interface SetABOverrideMessage extends DevtoolsMessage {
    type: MessageEventType.OVERRIDE_AB_SET;
    payload?: {
        experiment: Experiment;
        variationId: number;
    };
}
export interface SetFFOverrideMessage extends DevtoolsMessage {
    type: MessageEventType.OVERRIDE_FF_SET;
    payload?: {
        experiment: Experiment;
        variationId: number;
    };
}
export interface ResetABOverrideMessage extends DevtoolsMessage {
    type: MessageEventType.OVERRIDE_AB_RESET;
    payload?: {
        experiment: Experiment;
    };
}
export interface ResetFFOverrideMessage extends DevtoolsMessage {
    type: MessageEventType.OVERRIDE_FF_RESET;
    payload?: {
        experiment: Experiment;
    };
}
export interface ResetAllABOverrideMessage extends DevtoolsMessage {
    type: MessageEventType.OVERRIDE_AB_RESET_ALL;
}
export interface ResetAllFFOverrideMessage extends DevtoolsMessage {
    type: MessageEventType.OVERRIDE_FF_RESET_ALL;
}
export declare const isDevtoolsMessage: (message: any) => message is DevtoolsMessage;
export declare const isCloseMessage: (message: unknown) => message is CloseMessage;
export declare const isInitMessage: (message: unknown) => message is InitMessage;
export declare const isUserMessage: (message: unknown) => message is UserMessage;
export declare const isGetABDecisionMessage: (message: unknown) => message is GetABDecisionMessage;
export declare const isGetFFDecisionMessage: (message: unknown) => message is GetFFDecisionMessage;
export declare const isGetABOverrideMessage: (message: unknown) => message is GetABOverrideMessage;
export declare const isGetFFOverrideMessage: (message: unknown) => message is GetFFOverrideMessage;
export declare const isSetABOverrideMessage: (message: unknown) => message is SetABOverrideMessage;
export declare const isSetFFOverrideMessage: (message: unknown) => message is SetFFOverrideMessage;
export declare const isResetABOverrideMessage: (message: unknown) => message is ResetABOverrideMessage;
export declare const isResetFFOverrideMessage: (message: unknown) => message is ResetFFOverrideMessage;
export declare const isResetAllABOverrideMessage: (message: unknown) => message is ResetAllABOverrideMessage;
export declare const isResetAllFFOverrideMessage: (message: unknown) => message is ResetAllFFOverrideMessage;
